rm(list=ls(all=T))
library(qvalue) 

##########################################################
path="E:/CRC upload"  # please set working directory     #
##########################################################

setwd(path)
if(!file.exists("output")) dir.create("output")
#raw data path
path.raw="input/Dataset1_1.txt"
data.raw=read.csv(file=path.raw,header=T)
#panel data path
path.panel="input/Dataset1_2.txt"  ####
panel.raw=read.csv(file=path.panel,sep="\t",header=F)

#include panel data
gene.panel=vector("list",dim(panel.raw)[1])
measure.type.data=numeric(dim(panel.raw)[2])
name.string=NULL
for(i in 1:dim(panel.raw)[1]) {
   gene.panel[[i]]=c(as.character(panel.raw$V1[i]),as.character(panel.raw$V2[i]))
   name.string=rbind(name.string,paste(as.character(panel.raw$V1[i])," & ",as.character(panel.raw$V2[i]),sep=""))
}



T.index=which(substr(names(data.raw),1,1)=="T")
N.index=which(substr(names(data.raw),1,1)=="N")
gene.T.n=length(T.index)
gene.N.n=length(N.index)
gene.name=as.character(data.raw[,1])
gene.data=data.raw[,(2:dim(data.raw)[2])]
fold.change=log2(2)   ##Setting initial value##

Ratio.fun=function(dataT,dataN){
   mean.N=mean(2^dataN)
   log.r=dataT-log2(mean.N)
   return(log.r)
}
gene.perm.index=function(gene.T.n,gene.N.n,times){
   mix.gene.n=gene.T.n+gene.N.n
   index.m=matrix(0,ncol=mix.gene.n,nrow=times)
   for(i in 1:times) index.m[i,]=sample(1:mix.gene.n,mix.gene.n,replace=F)
   return(index.m)
}
### measure type up-up=1 up-down=2 down-up=3 down-down=4
Permutation.s.test=function(gene1.t,gene1.w,gene2.t,gene2.w,index.m){
   gene.T.n=length(gene1.t)
   gene.N.n=length(gene1.w)
   mix.gene1=c(gene1.t,gene1.w)
   mix.gene2=c(gene2.t,gene2.w)
   mix.gene.n=length(mix.gene1)
   Ratio.1=Ratio.fun(gene1.t,gene1.w)
   Ratio.2=Ratio.fun(gene2.t,gene2.w)
   obs_prob.temp=numeric(4)
   obs_prob.temp[1]=sum(Ratio.1>=fold.change & Ratio.2>=fold.change)/gene.T.n
   obs_prob.temp[2]=sum(Ratio.1>=fold.change & Ratio.2<=-fold.change)/gene.T.n
   obs_prob.temp[3]=sum(Ratio.1<=-fold.change & Ratio.2>=fold.change)/gene.T.n
   obs_prob.temp[4]=sum(Ratio.1<=-fold.change & Ratio.2<=-fold.change)/gene.T.n
   names(obs_prob.temp) <- c("up,up","up,down","down,up","down,down")
   obs_prob=max(obs_prob.temp)
   measure.type=which.max(obs_prob.temp)   
   times=dim(index.m)[1]
   perm_prob=numeric(times)
   for(i in 1:times){
      index=index.m[i,]
      Ratio.1=Ratio.fun(mix.gene1[index[1:gene.T.n]],mix.gene1[index[(gene.T.n+1):mix.gene.n]])
      Ratio.2=Ratio.fun(mix.gene2[index[1:gene.T.n]],mix.gene2[index[(gene.T.n+1):mix.gene.n]])
      if (measure.type==1) perm_prob[i]=sum(Ratio.1>=fold.change & Ratio.2>=fold.change)/gene.T.n
      if (measure.type==2) perm_prob[i]=sum(Ratio.1>=fold.change & Ratio.2<=-fold.change)/gene.T.n
      if (measure.type==3) perm_prob[i]=sum(Ratio.1<=-fold.change & Ratio.2>=fold.change)/gene.T.n
      if (measure.type==4) perm_prob[i]=sum(Ratio.1<=-fold.change & Ratio.2<=-fold.change)/gene.T.n
   }
   p.value=(sum(perm_prob>=obs_prob)+1)/(times+1)
   return(list(p.hat=obs_prob.temp,p.value=p.value,perm_prob=perm_prob))
}
record.time=numeric(dim(panel.raw)[1])
B.times=9999   ##Setting initial value##
index.m=gene.perm.index(gene.T.n,gene.N.n,times=B.times)
record.panel=vector("list",dim(panel.raw)[1])
for(i in 1:dim(panel.raw)[1]){
   gene1.index=which(gene.name==gene.panel[[i]][1])
   gene2.index=which(gene.name==gene.panel[[i]][2])
   gene1.T.data=as.numeric(data.raw[gene1.index,T.index])
   gene1.N.data=as.numeric(data.raw[gene1.index,N.index])
   gene2.T.data=as.numeric(data.raw[gene2.index,T.index])
   gene2.N.data=as.numeric(data.raw[gene2.index,N.index])
   record.panel[[i]]=Permutation.s.test(gene1.t=gene1.T.data,gene1.w=gene1.N.data,gene2.t=gene2.T.data,gene2.w=gene2.N.data,index.m=index.m)
}


p.value=numeric(dim(panel.raw)[1])
p.hat=matrix(nrow=dim(panel.raw)[1],ncol=4)
colnames(p.hat) <- c("up,up","up,down","down,up","down,down")
for(i in 1:dim(panel.raw)[1]) {
   p.value[i]=record.panel[[i]]$p.value
   p.hat[i,]=record.panel[[i]]$p.hat
}
#FDR.value=p.adjust(p.value,method="BH")
FDR.value=qvalue(p.value)$qvalues  
output.data=cbind(p.hat,p.vaule=p.value,FDR=FDR.value)
rownames(output.data)=name.string
write.csv(output.data,file=paste("output/output of fractions of gene expression patterns.csv",sep=""))



